import hashlib
import re
from paytring.resources import Paytring

class Utility(Paytring):
    
    
    def create_hash(self, body):
    
        """Create hash for given body and key"""
        try:
            if len(body.keys()) != 0:
                keys = sorted(body.keys())
                value = [body[key] for key in keys]
                value = '|'.join(value) + '|'
                value += self.secret
                return hashlib.sha512(value.encode('utf-8')).hexdigest()
            else:
                raise Exception('Invalid Payload')
        except Exception as e:
            raise Exception(str(e))

    def validate_email(self, email) -> bool:
        if not isinstance(email, str):
            raise Exception('Invalid email')
        
        regex = '^[a-z0-9]+[\._]?[a-z0-9]+[@]\w+[.]\w{2,3}$'
        if(re.search(regex, email)):
            return True
        raise Exception('Invalid email')

    def validate_amount(self, amount) -> bool:
        if not isinstance(amount, str):
            raise Exception('Invalid amount')
        
        if amount.isnumeric():
            return True
        raise Exception('Invalid amount')

    def validate_phone(self, phone) -> bool:
        if not isinstance(phone, str):
            raise Exception('Invalid phone number')
        
        regex = re.compile("(0|91)?[6-9][0-9]{9}")
        if regex.match(phone):
            return True
        raise Exception('Invalid phone number')

    def validate_callback_url(self, callback_url) -> bool:
        if isinstance(callback_url, str):
            return True
        raise Exception('Invalid callback url')

    def validate_notes(self, notes) -> bool:
        if isinstance(notes, dict):
            return True
        raise Exception('Invalid notes')

    def vaidate_customer_info(self, customer_info) -> bool:
        if customer_info.keys() == {'cname', 'email', 'phone'}:
            return True
        raise Exception('Insufficient customer info')

    def validate_receipt(self, receipt_id) -> bool:
        if isinstance(receipt_id, str):
            return True
        raise Exception('Invalid receipt id')
    
    def validate_currency(self, currency) -> bool:

        if not isinstance(currency, str):
            raise Exception('Invalid Currency Format number')
        pattern = r'^(INR|USD)$'
        return bool(re.match(pattern, currency))
    
    def validate_order(self, order_id) -> bool:
        if isinstance(order_id, str):
            return True
        raise Exception('Invalid order id')




class URL:
    BASE_URL = "https://api.paytring.com/api/v1/"
    ORDER_CREATE = "https://api.paytring.com/api/v1/order/create"
    FETCH_ORDER = "https://api.paytring.com/api/v1/order/fetch"
    FETCH_ORDER_BY_RECIEPT = "https://api.paytring.com/api/v1/order/fetch/receipt"
    REFUND = "https://api.paytring.com/api/v1/order/refund"


import os

class Paytring:
    """
    Intialisation of Key and Secret 
    """
    def __init__(self) -> None:
        
        self.key = os.getenv('key')
        self.secret = os.getenv('secret')
        if not self.key or not self.secret:
            raise Exception('SDK key and secret must be provided.')
        
    


from setuptools import setup, find_packages

with open("Paytring/README.md", "r") as f:
    long_description = f.read()

setup(
    name="paytring",
    version="1.0.1",
    description="A SDK which helps to create, fetch or refund an order on Paytring",
    package_dir={"": "Paytring"},
    packages=find_packages(where="Paytring"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/paytring/python-sdk",
    author="Paytirng",
    author_email="developer@paytring.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
    ],
    install_requires=["requests==2.31.0","urllib3==2.0.2"],
    python_requires=">=3.10",
)


from paytring.constant.url import URL
from paytring.resources.paytring import Paytring
from paytring.utility.utility import Utility
import requests
import base64

class Order(Paytring):

    def __init__(self):
        super().__init__()
        self.order_create_url = URL.ORDER_CREATE
        self.order_fetch_url = URL.FETCH_ORDER
        self.order_fetch_by_receipt_url = URL.FETCH_ORDER_BY_RECIEPT
        self.refund_url = URL.REFUND
        self.utility_obj = Utility()

    def create(self, receipt_id, amount, callback_url,customer_info, currency):
        """
        Use to create an Order on Paytring

        Args(type=string):
            'receipt' : Receipt Id for the order
            'amount' :  Amount of Order
            'callback_url' : The URL where the PAYTRING will send success/failed etc. response.
            'customer_info' : Info. about Customer.
            'currency' : Currency in which the amount is entered 

        Returns:
            Order Dict created for given reciept ID
        """
        try:
            self.utility_obj.vaidate_customer_info(customer_info)
            self.utility_obj.validate_email(customer_info['email'])
            self.utility_obj.validate_phone(customer_info['phone'])
            self.utility_obj.validate_amount(amount)
            self.utility_obj.validate_callback_url(callback_url)
            self.utility_obj.validate_receipt(receipt_id)
            self.utility_obj.validate_currency(currency.upper())
        
            payload = {
                "key": self.key,
                "receipt_id": receipt_id,
                "amount": amount,
                "callback_url": callback_url,
                "cname": customer_info['cname'],
                "email": customer_info['email'],
                "phone": customer_info['phone'],
                "currency" : currency
            }

            hash = self.utility_obj.create_hash(payload)
            payload['hash'] = hash

            response = requests.post(self.order_create_url, payload)
            response = response.json()
            if response['status'] == True:
                    if 'url' in response.keys():
                        response['url'] = base64.b64decode(response['url']).decode('utf-8')
                    return {"response": response}
            return {"response": response}
        except Exception as e:
             return {"response" : str(e)}
        
    def fetch(self, order_id):
        """
        Use to fetch an Order on Paytring throu

        Args: 
            order_id : Id for which order object has to be retrieved
        
        Returns:
            Order Dict for given order_id
        """
        try:
            self.utility_obj.validate_order(order_id)

            payload = {
                "key": self.key,
                "id": order_id
            }
            hash = self.utility_obj.create_hash(payload)
            payload['hash'] = hash
            response = requests.post(self.order_fetch_url, payload)
            response = response.json()
            if response['status'] == True:
                return {"response": response}
            return {"response": response}
        except Exception as e:
            return {"response": str(e)}
        
    def fetch_by_receipt_id(self, receipt_id):
        """
        Use to fetch an Order on Paytring by receipt-id

        Args: 
            receipt_id : Id for which order object has to be retrieved
        
        Returns:
            Order Dict for given receipt_id
        """

        try:
            self.utility_obj.validate_receipt(receipt_id)

            payload = {
                "key": self.key,
                "id": receipt_id
            }
            hash = self.utility_obj.create_hash(payload)
            payload['hash'] = hash
            response = requests.post(self.order_fetch_by_receipt_url, payload)
            response = response.json()
            if response['status'] == True:
                return {"response": response}
            return {"response": response}
        except Exception as e:
            return {"response": str(e)}
        
    def refund(self, order_id):
        """
        Use to intaite refund on Paytring by order-id

        Args: 
            order_id : Id for which refund is to be intiated
        
        Returns:
            Dict containing 'status' and 'message'
        """

        try:
            payload = {
                "key": self.key,
                "id": order_id
            }
            hash = self.utility_obj.create_hash(payload)
            payload['hash'] = hash
            response = requests.post(self.refund_url, payload)
            response = response.json()
            if response['status'] == True:
                return {"response": response}
            return {"response": response}
        except Exception as e:
            return {"response": str(e)}
